% script.m
% Main M-file for Particel Filter problem.
% 
% Matlab script that is used to simulate the truth model, call the
% estimator and show the results.
%
% Class:
% Introduction to Recursive Filtering and Estimation
% Spring 2010
% Programming Exercise 2
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Raffaello D'Andrea, Angela Schoellig
% aschoellig@ethz.ch
%
% --
% Revision history
% [25.05.10, AS]    first version
%
%

% clear workspace and command window
clear;
clc;
close all;


%% Setup
% Define the physical constants used in the simulation.  These are
% accessible to the estimator.
physConst = PhysicalConstants();

% Define the simulation constants used in the simulation.  Not accessible
% to the estimator.
simConst = SimulationConstants();

% Set the random number generator state.
% Uncomment to make results reproducable.
%rand('state',0);



%% Initialize Variables

%%%
% The following are invisible to the estimator
%%%

% The bias p of going CCW, as a fraction of \bar{s}
relativeBias = simConst.relativeBias;

% The number of time steps in simulation
T = simConst.simulationTime;

% The object location 
loc = zeros(T+1,1);

% The initial location of the object, a real number between 0 and 2*pi
loc(1) = simConst.initialObjectPos;


%%%
% The following are visible to the estimator
%%%

% The distance sensor.  Set to "inf" when no reading is made.
distSensor = zeros(T,1);

% The half-plane sensor.  Set to "inf" when no reading is made.
halfPlaneSensor = zeros(T,1);



%% Simulation: Generate data
% Simulation of the system to obtain measurements and the true
% states.  The measurements will be visible to the estimator.

% Measurements are acquired at randomized time intervals whose length is
% between the minimum and maximum length specified in SIMCONST.

% The next distance reading: if the simulation time is DISTANCETIME, a
% new distance measurement is acquired.
distanceTime = round(UniformMinMax(simConst.sampleDistanceMin,simConst.sampleDistanceMax));

% The next half-plane reading: if the simulation time is HALFPLANETIME, a
% new half-plane measurement is acquired.
halfPlaneTime =  round(UniformMinMax(simConst.sampleHalfPlaneMin,simConst.sampleHalfPlaneMax));


% Simulate the system, and store object position and sensor information.
for t = 1:T
    
    % Process dynamics
    loc(t+1) = mod(loc(t) + relativeBias*physConst.StepSize + Uniform(physConst.StepSize), 2*pi);
    
    % The sensor reading.  The default is no measurement
    distSensor(t) = inf;
    halfPlaneSensor(t) = inf;
    
    % See if a distance reading is made
    if (t >= distanceTime)
        distanceTime = t + round(UniformMinMax(simConst.sampleDistanceMin,simConst.sampleDistanceMax));
        
        % The physical location of the object is on the unit circle
        xLoc = cos(loc(t+1));
        yLoc = sin(loc(t+1));
        
        % Measured distance to the object
        distSensor(t) = sqrt( (physConst.SensorLoc - xLoc)^2 + yLoc^2) + Uniform(physConst.SensorErr);  
    end
    
    % See if a half-plane reading is made.      
    if (t >= halfPlaneTime)
        halfPlaneTime = t + round(UniformMinMax(simConst.sampleHalfPlaneMin,simConst.sampleHalfPlaneMax));
        
        % Half plane of object
        if (loc(t+1) >= pi)
            halfPlaneSensor(t) = -1;
        else
            halfPlaneSensor(t) = 1;
        end      
    end 

end



%% Estimator
% Run the estimator.

% Initialize the estimator.  
estState = []; 
locationParticles = zeros(T+1,physConst.NumParticles);
biasParticles = zeros(T+1,physConst.NumParticles);

[locationParticles(1,:), biasParticles(1,:), estState] = Estimator(estState,zeros(1,2),0,physConst);

% Call the estimator for each time step.
for t = 1:T
    [locationParticles(t+1,:), biasParticles(t+1,:), estState] = Estimator(estState,[distSensor(t),halfPlaneSensor(t)],t,physConst);
end




%% The results
% Plot of location particles.

% Show phantom object.  If set to 1, the phantom object is displayed.  The
% phantom object is simply the reflection of the object about the
% horizontal axis; there is no way for the distance sensor to differentiate
% between the two.
SHOW_PHANTOM = 1;
%SHOW_PHANTOM = 0;

figure(1)
for t = 0:T  
    % Plot the results
    numberBins = physConst.NumParticles;
    S = physConst.StepSize;
    
   % The biases
    subplot(2,1,1)
    biasHist = histc(biasParticles(t+1,:),(2*S*(0:numberBins-1)/numberBins - S));
    bar( (2*S*(0:numberBins-1)/numberBins - S), biasHist/max(biasHist));  
    axis([-S S 0 1]);    
    title('BIAS HISTOGRAM');
    xlabel('BIAS (RAD)');
    hold on
    dummy = plot(relativeBias*S,1,'b*');
    set(dummy,'MarkerSize',20);
    set(dummy,'LineWidth',3);
    if (SHOW_PHANTOM)
        dummy = plot(-relativeBias*S,1,'m*');
        set(dummy,'MarkerSize',20);
        set(dummy,'LineWidth',3);
    end
    text(-S,1.1,[num2str(100*t/simConst.simulationTime),'%']);
    hold off    
    
    
    % The location
    subplot(2,1,2)
    % The location
    locHist = histc(locationParticles(t+1,:),2*pi*(0:numberBins-1)/numberBins);
    bar(2*pi*(0:numberBins-1)/numberBins, locHist/max(locHist));    
    axis([0 2*pi 0 1]); 
    title('POSITION HISTOGRAM');
    xlabel('POSITION (RAD)');
    hold on
    dummy = plot(loc(t+1),1,'b*');
    set(dummy,'MarkerSize',20);
    set(dummy,'LineWidth',3);
    if (SHOW_PHANTOM)
        dummy = plot(mod(-loc(t+1) + 2*pi,2*pi),1,'m*');
        set(dummy,'MarkerSize',20);
        set(dummy,'LineWidth',3);
    end  
    hold off  
    pause(0.01);
end

